@@ -1,4 +1,4 @@
-##########################################################################
+#########################################################################
 ## All portions of this code are copyright (c) 2003,2004 nethype GmbH   ##
 ##########################################################################
 ## Using, reading, modifying or copying this code requires a LICENSE    ##
@@ -43,6 +43,7 @@ use Carp;
 
 use PApp::Config qw(DBH $DBH $Database); DBH;
 
+use PApp ();
 use PApp::Env;
 use PApp::SQL;
 use PApp::Event;
@@ -52,7 +53,7 @@ use PApp::Callback ();
 use PApp::Exception;
 use PApp::I18n ();
 
-# load these so their callbacks cna be registered
+# load these so their callbacks can be registered
 # TODO: should be done by papp proper
 use PApp::EditForm ();
 use PApp::XPCSE ();
@@ -66,6 +67,11 @@ our $env; # current content::environment
 
 our %temporary; # used by the "temporary" attribute type
 
+BEGIN {
+   *DEVEL_TRACE = sub () { 0 }
+      unless defined &DEVEL_TRACE;
+}
+
 BEGIN {
    # I was lazy, all the util xs functions are in PApp.xs
    require XSLoader;
@@ -174,7 +180,7 @@ sub init_paths {
 
    for (values %obj_cache) {
       for (@$_) {
-         $_ 
+         $_
             and $_->{_paths} =
                sql_fetch "select paths from obj where gid = ? and paths & (1 << ?) <> 0",
                          $_->{_gid}, $_->{_path};
@@ -219,17 +225,17 @@ our %sqlcol_dbi_type = (
 
 sub prepare_papp_dbh {
   my $dbh = shift;
-  
+
   for (@sqlcol) {
-    $fetch_sqlcol{$_} = 
+    $fetch_sqlcol{$_} =
       $dbh->prepare ("select data from $_ where id = ? and type = ?");
 
-    my $st = $storefetch_sqlcol{$_} = 
+    my $st = $storefetch_sqlcol{$_} =
       $dbh->prepare ("select data <=> ? from $_ where id = ? and type = ?");
-      
+
     $st->bind_param (1, undef, { TYPE => $Agni::sqlcol_dbi_type{$_} });
 
-    my $st = $storeupdate_sqlcol{$_} = 
+    my $st = $storeupdate_sqlcol{$_} =
       $dbh->prepare ("update $_ set data = ? where (not (data <=> ?)) and id = ? and type = ?");
 
     $st->bind_param (1, undef, { TYPE => $Agni::sqlcol_dbi_type{$_} });
@@ -282,7 +288,7 @@ sub newpath($) {
 
 # return the pathid of the staging path corresponding to the given path
 sub staging_path($) {
-   defined $_[0] and defined $pathname[$_[0]] 
+   defined $_[0] and defined $pathname[$_[0]]
       or die "staging_path called without a pathid\n";
    (my $path = $pathname[$_[0]]) =~ s{/(staging/)?$}{/staging/};
    newpath $path unless exists $pathid{$path};
@@ -293,7 +299,7 @@ sub staging_path($) {
 
 # the reverse to staging_path
 sub commit_path($) {
-   defined $_[0] and defined $pathname[$_[0]] 
+   defined $_[0] and defined $pathname[$_[0]]
       or die "staging_path called without a pathid\n";
    (my $path = $pathname[$_[0]]) =~ s{/staging/$}{/};
    newpath $path unless exists $pathid{$path};
@@ -537,15 +543,18 @@ sub compile {
    }{
       my ($type, $content) = ($1, $2);
       if ($type eq $objtag_obj) {
-         "+(\$Agni::obj_cache{'$content'}[$PACKAGE->{_path}] || obj '$content')";
+         # we have to load delayed to be able to bootstrap, currently
+         #"+(\$Agni::obj_cache{'$content'}[$PACKAGE->{_path}] || obj '$content')"
+         "+(state \$__ = obj '$content')"
       } elsif ($type eq $objtag_obj_gid) {
          "'$content'";
       } else {
          warn "unknown method tag " . ((ord $type) - (ord $objtag_type_lo) + 2) . " with content '$content', maybe you need a newer version of agni?\n";
          "";
       }
-   }ogex;
+   }ogex; # " vim fix
 
+   use 5.010;
    use strict qw(vars subs);
    local $SIG{__DIE__};
 
@@ -698,7 +707,7 @@ sub update_isa_class($$) {
 
       if ($old eq ref $self->{_isa}) {
          # has no own methods or similar, so inherits package
-       
+
          if ($old ne $new) {
             for (values %obj_cache) {
                agnibless $obj, $new
@@ -810,6 +819,8 @@ sub update_class($) {
 
    $id or return undef;
 
+   ::trace_update_class_enter ($self) if DEVEL_TRACE;
+
    # to avoid endless recursion, set the object before loading the isa object
    # (not a problem under normal circumstances)
    $obj_cache{$self->{_gid}}[$self->{_path}] = $self;
@@ -877,7 +888,7 @@ sub update_class($) {
                   compile_method_perl $self, $name, $args, pcode2perl perl2pcode utf8_on $data;
                } else {
                   # non-perl-method, store for later use
-    
+
                   # plant a bomb
                   my $class = ref $self;
                   *{"$class\::$name"} = sub { die "non-bootstrap method $class->$name ($args) called during bootstrap" };
@@ -891,7 +902,7 @@ sub update_class($) {
             _obj_cache_set $self, $type, $data if $superclass eq $OID_DATA_STRING;
 
             # plant a bomb, so other accesses than fetch die
-            _obj_member_add $self, $name, bless { _gid => $type }, 
+            _obj_member_add $self, $name, bless { _gid => $type },
                                            "non-bootstrap data access during bootstrap ($self->{_path}/$self->{_gid}\{$type=$name}";
 
             $postponed->{$type} = $data;
@@ -930,6 +941,8 @@ sub update_class($) {
 
    rmagical_on $self;
 
+   ::trace_update_class_leave ($self) if DEVEL_TRACE;
+
    $self
 }
 
@@ -1081,11 +1094,12 @@ sub obj2name($) {
 =item commit_objs [$gid, $src_path, $dst_path], ...
 
 Commit (copy) objects from one path to another. If C<$dst_path> is
-undefined or missing, deletes the object.
+undefined or missing, deletes the instance (making higher-path instances
+visible again).
 
-Currently, C<$src_path> must be the "topmost" path of one object instace,
-undefined behaviour will result if the instance exists in a path higher
-than C<$src_path>.
+Currently, C<$src_path> must be the "topmost" path of one object
+instance. Object instances that are also visible in parent paths are
+skipped.
 
 It returns a html fragment describing it's operations.
 
@@ -1157,7 +1171,7 @@ sub commit_objs {
 
                   # can't happen anymore?
                   $id != $obj_id or croak "FATAL, pls report! commit_objs: src_path $src not the highest path of object $obj_gid";
-                            
+
                   if ($id) {
                      # remove it from the target path
                      if (andnot64 $paths, $subpathmask[$dst]) {
@@ -1408,7 +1422,7 @@ sub find_dead_objects {
                              inner join d_int on (d_int.type = $OID_ISA and d_int.data = iobj.gid)
                              inner join obj on (obj.id = d_int.id)
                           where obj.id in ($in) and obj.paths & iobj.paths <> 0";
-            
+
          for my $sqlcol (@sqlcol) {
             # now fetch all attrs of the objects, mark them alive and resolve forward references
             my $st = sql_exec \my($id, $tgid, $tid, $paths),
