@@ -65,7 +65,7 @@ use PApp::Config;
 BEGIN {
    use base 'Exporter';
 
-   $VERSION = 2.0;
+   $VERSION = 2.1;
    @EXPORT = qw();
    @EXPORT_OK = qw(
          open_translator
@@ -80,9 +80,11 @@ BEGIN {
 
 my ($iso3166, $iso639, $locale) = do {
    local $/;
-   split /^__SPLIT__/m, utf8_on <DATA>;
+   split /^__SPLIT__/m, utf8_on <DATA>
 };
 
+close DATA;
+
 {
    sub iso639          { [map [split /\s+/, $_, 3], split /\n/, $iso639 ] }
    sub iso639_a2_a3    { $iso639 =~ /^(...)\t\Q$_[0]\E\t/m ? $1 : $_[0] }
