@@ -123,7 +123,7 @@ use PApp::DataRef ();
 use Convert::Scalar qw(:utf8 weaken);
 
 BEGIN {
-   our $VERSION = 2.0;
+   our $VERSION = 2.1;
 
    use base Exporter::;
 
@@ -146,7 +146,7 @@ BEGIN {
          $userid $sessionid
          reload_p switch_userid getuid
 
-         dprintf dprint echo capture $request 
+         dprintf dprint echo capture $request
          
          N_ language_selector preferences_url preferences_link
          $prefs $curprefs getpref setpref save_prefs
@@ -323,7 +323,7 @@ filehandle or whatsoever with these names.
 
 =item $request [read-only]
 
-The Apache request object (L<Apache>), the same as returned by C<Apache->request>.
+The Apache request object (L<Apache>), the same as returned by C<< Apache->request >>.
 
 =item %state [read-write, persistent]
 
@@ -823,7 +823,7 @@ replacement text (initially empty if not specified). At page output time
 any fixup markers in the document are replaced by this scalar.
 
 The initial content can also be a code reference which will be evaluated
-at marker output time.
+at page output time.
 
 =item $ref = insert_fixup [$initial_content]
 
@@ -1111,7 +1111,7 @@ goodies (like redirected STDOUT). It has to return one of the status codes
 
 If you want to output something in an upcall, try to use this sequence:
 
-   abort_with {
+   PApp::send_upcall {
       content_type "text/html";
       $request->status ("401");
       $request->header_out ("WWW-Authenticate" => "Basic realm=\"$realm\"");
@@ -1789,7 +1789,8 @@ sub _handler {
       local $SIG{__DIE__}  = \&PApp::Exception::diehandler;
       local $SIG{__WARN__} = \&warnhandler;
 
-      $DBH = DBH;
+      local $PApp::SQL::Database = $PApp::Config::Database;
+      local $PApp::SQL::DBH      = $DBH = DBH;
 
       %P = %arguments = ();
       _set_params PApp::HTML::parse_params $request->query_string;
@@ -1856,14 +1857,14 @@ sub _handler {
 
          set_alternative $state{papp_alternative}[$alternative];
 
-         $papp = $papp{$state{papp_appid}}
-                 or fancydie "Application not mounted", $location,
-                             info => [appid => $state{papp_appid}];
+#         $papp = $papp{$state{papp_appid}}
+#                 or fancydie "Application not mounted", $location,
+#                             info => [appid => $state{papp_appid}];
 
       } else {
          start_session;
 
-         $state{papp_appid} = $papp->{appid};
+#         $state{papp_appid} = $papp->{appid};
 
          #$modules = $pathinfo =~ m%/(.*?)/?$% ? modpath_thaw $1 : {}; #d#
 
