@@ -49,7 +49,7 @@ use Exporter;
 BEGIN {
    our @ISA = (PApp::Base::);
    unshift @PApp::ISA, __PACKAGE__;
-   our $VERSION = 2.0;
+   our $VERSION = 2.1;
 }
 
 =head2 Functions
@@ -202,47 +202,55 @@ sub warn {
    print STDERR $_[1];
 }
 
+sub new_from {
+   my ($class, $env) = @_;
+
+   $class = bless {
+      document_root => $env->{DOCUMENT_ROOT},
+      path_info     => $env->{PATH_INFO},
+      query_string  => $env->{QUERY_STRING},
+      #uri           => $env->{SCRIPT_URL} || $env->{SCRIPT_URI} || "$env->{SCRIPT_NAME}$env->{PATH_INFO}",
+      name          => $env->{SCRIPT_NAME},
+      method        => $env->{REQUEST_METHOD},
+      filename      => $env->{PATH_TRANSLATED},
+      server_admin  => $env->{SERVER_ADMIN},
+      server_hostname=>$env->{SERVER_NAME},
+      server_admin  => $env->{SERVER_ADMIN},
+      protocol      => $env->{SERVER_PROTOCOL},
+      headers_in    => {
+         "Content-Type"		=> $env->{CONTENT_TYPE},
+         "Content-Length"	=> $env->{CONTENT_LENGTH},
+         # should just copy ALL HTTP_*-headers
+         "Accept-Language"	=> $env->{HTTP_ACCEPT_LANGUAGE},
+         "Accept"		=> $env->{HTTP_ACCEPT},
+         "Accept-Charset"	=> $env->{HTTP_ACCEPT_CHARSET},
+         "Accept-Encoding"	=> $env->{HTTP_ACCEPT_ENCODING},
+         "Cookie"		=> $env->{HTTP_COOKIE},
+         "Host"			=> $env->{HTTP_HOST},
+         "Connection"		=> $env->{HTTP_CONNECTION},
+         "User-Agent"		=> $env->{HTTP_USER_AGENT},
+      },
+      connection    => (bless {
+         remote_host		=> $env->{REMOTE_HOST},
+         remote_ip		=> $env->{REMOTE_ADDR},
+         remote_port		=> $env->{REMOTE_PORT},
+         local_ip		=> $env->{SERVER_ADDR},
+         local_port		=> $env->{SERVER_PORT},
+         remote_logname		=> $env->{REMOTE_IDENT},
+         auth_type		=> $env->{AUTH_TYPE},
+         user			=> $env->{REMOTE_USER},
+      }, PApp::CGI::Connection),
+      @_,
+   }, $class;
+
+   $class
+}
+
 sub new {
    my $class = shift;
 
    if ($ENV{GATEWAY_INTERFACE} =~ /^CGI\//) {
-      $class = bless {
-         document_root => $ENV{DOCUMENT_ROOT},
-         path_info     => $ENV{PATH_INFO},
-         query_string  => $ENV{QUERY_STRING},
-         #uri           => $ENV{SCRIPT_URL} || $ENV{SCRIPT_URI} || "$ENV{SCRIPT_NAME}$ENV{PATH_INFO}",
-         name          => $ENV{SCRIPT_NAME},
-         method        => $ENV{REQUEST_METHOD},
-         filename      => $ENV{PATH_TRANSLATED},
-         server_admin  => $ENV{SERVER_ADMIN},
-         server_hostname=>$ENV{SERVER_NAME},
-         server_admin  => $ENV{SERVER_ADMIN},
-         protocol      => $ENV{SERVER_PROTOCOL},
-         headers_in    => {
-            "Content-Type"	=> $ENV{CONTENT_TYPE},
-            "Content-Length"	=> $ENV{CONTENT_LENGTH},
-            # should just copy ALL HTTP_*-headers
-            "Accept-Language"	=> $ENV{HTTP_ACCEPT_LANGUAGE},
-            "Accept"		=> $ENV{HTTP_ACCEPT},
-            "Accept-Charset"	=> $ENV{HTTP_ACCEPT_CHARSET},
-            "Accept-Encoding"	=> $ENV{HTTP_ACCEPT_ENCODING},
-            "Cookie"		=> $ENV{HTTP_COOKIE},
-            "Host"		=> $ENV{HTTP_HOST},
-            "Connection"	=> $ENV{HTTP_CONNECTION},
-            "User-Agent"	=> $ENV{HTTP_USER_AGENT},
-         },
-         connection    => (bless {
-            remote_host		=> $ENV{REMOTE_HOST},
-            remote_ip		=> $ENV{REMOTE_ADDR},
-            remote_port		=> $ENV{REMOTE_PORT},
-            local_ip		=> $ENV{SERVER_ADDR},
-            local_port		=> $ENV{SERVER_PORT},
-            remote_logname	=> $ENV{REMOTE_IDENT},
-            auth_type		=> $ENV{AUTH_TYPE},
-            user		=> $ENV{REMOTE_USER},
-         }, PApp::CGI::Connection),
-         @_,
-      }, $class;
+      $class = $class->new_from (\%ENV);
    } else {
       die "cgi environment currently required\n";
    }
@@ -334,7 +342,7 @@ sub content {
 
 sub read {
    CORE::read STDIN,
-              $_[1], 
+              $_[1],
               ($_[2] > $_[0]->{headers_in}{"Content-Length"}
                   ? $_[0]->{headers_in}{"Content-Length"}
                   : $_[2]),
@@ -367,19 +375,22 @@ sub send_http_header {
       my $status = delete $self->{headers_out}{Status} || 200;
       push @hdr, "HTTP/1.0 $status Status $status\015\012Connection: close\015\012";
    }
+
    while (my ($h, $v) = each %{$self->{headers_out}}) {
       push @hdr, "$h: $v\015\012";
    }
+
    for (@{ $self->{set_cookie} }) {
       push @hdr, "Set-Cookie: $_\015\012";
    }
-   print @hdr, "\015\012";
+
+   $self->print (@hdr, "\015\012");
 }
 
 sub print {
    use bytes;
    my $self = shift;
-   print @_;
+   syswrite STDOUT, join "", @_
 }
 
 =item internal_redirect "relative-url"
@@ -441,8 +452,8 @@ package PApp::CGI::Connection;
 =head2 The PApp::CGI::Connection Class
 
 This class implements the connection object returned by
-C<$request->conenction).  The methods in this class are mostly identical
-to the methods supported by the conenction object in the L<Apache> module.
+C<$request->connection).  The methods in this class are mostly identical
+to the methods supported by the connection object in the L<Apache> module.
 
 =over 4
 
